import numpy as np
import random 
import copy 
import time
import json
import scipy.io

import I_trans
import O_trans


'''
Key: SQEIR
'''

#basic_parameter or argument

Pi = 0.001					#初始I比例（调节参数）
beita1 = 0.3				#S到被I感染到E的概率
beita = 0.8					#从S出去的总数
beita2 = beita-beita1		#易染者被隔离的概率
qi = 0.0005					#隔离者隔离期内被确诊概率
sige = 0.33 				#E发作率
gama = 0.3					#I到R的概率（调节参数）
alpha = 0.05				#I到D死亡率（调节参数）
Ei = 14						#Q隔离时长

force_in = 1				#社群内部管控力度（1-force_in）
force_out = 1				#社群之间管控力度（1-force_out）
time_in = 100				#社群内部开始管控的时间
time_out = 100				#社群之间开始管控的时间

iterations = 100			#迭代次数（调节参数,一代为一天）
timeses = 50				#跑了轮数，取平均值

choose = [0,1,3,4,6,7]
#0是隔离Q，1是易感S，3是潜伏E，4是感染I，6是恢复R，7是死亡D（仅作说明，不进参与计算）


'''
1、加载初始网络信息
'''
with open("Net.json") as f:
	Net = json.load(f)
len_nodes = len(Net)

#加载邻居
with open("Ori_neibors.json") as f:
	neibors = json.load(f)

#加载分组
with open("cluster.json") as f:
	cluster = json.load(f)


'''
2、选择生成新的潜伏期节点比例分布（c）or使用已有的潜伏期节点比例分布（l）
'''
task_choose = input("Choose load or creat the 潜伏者初始分布? enter ： l or c : ")
if task_choose == 'c':
	ori_itselves = np.zeros(len_nodes)				#创建每个节点的状态，长度为节点总数
	ori_itselves = ori_itselves.tolist()
	for n in range(len(ori_itselves)):
		p = np.random.random()
		if p < Pi:
			ori_itselves[n] = 4				#潜伏期
		else:
			ori_itselves[n] = 1				#易感者
	
	#储存初始节点状态分布
	name1 = 'Ori_itselves'
	name2 = '.json'
	file_name = name1 + name2
	with open(file_name, 'w') as f_obj:
		json.dump(ori_itselves, f_obj)
	
elif task_choose == 'l':
	#加载已生成的感染者分布
	file_name = 'Ori_itselves.json'
	with open(file_name) as f_obj:
		ori_itselves = json.load(f_obj)


'''
3、开始进入演化：
'''
Q_no = np.zeros(iterations)
S_no = np.zeros(iterations)
E_no = np.zeros(iterations)
I_no = np.zeros(iterations)
R_no = np.zeros(iterations)
D_no = np.zeros(iterations)

S_no_value = []
I_no_value = []

for times in range(timeses):
	print( str(time.strftime('%Y-%m-%d %H:%M:%S__',time.localtime(time.time())))\
	 + " times " + str(times) + " Start！" )

	Q1 = np.zeros(iterations)
	S1 = np.zeros(iterations)
	E1 = np.zeros(iterations)
	I1 = np.zeros(iterations)
	R1 = np.zeros(iterations)
	D1 = np.zeros(iterations)
	
	I_value_all = []
	E_days = [ 0 for i in range (len_nodes) ]		#创建潜伏期的天数记录列表
	itselves = copy.deepcopy(ori_itselves)
	
	for iteration in range(iterations) :			#iteration为传播天数
		for x in range(len_nodes):					#节点x的索引
			if itselves[x] == 1:					#若自身是易染者S
				if iteration < time_in and iteration < time_out :				#无管控
					itselves[x],E_days[x] = I_trans.no_force(x, \
					len_nodes, itselves, E_days, neibors, beita1)
				
				elif iteration >= time_in and iteration < time_out :			#仅组内管控
					itselves[x],E_days[x] = I_trans.in_force(x, len_nodes, \
					itselves, E_days, neibors, cluster, beita, beita1, force_in)
					
				elif iteration < time_in and iteration >= time_out :			#仅组间管控
					itselves[x],E_days[x] = I_trans.out_force(x, len_nodes, \
					itselves, E_days, neibors, cluster, beita, beita1, force_out)
					
				elif iteration >= time_in and iteration >= time_out :			#全管控
					itselves[x],E_days[x] = I_trans.two_force(x, len_nodes, \
					itselves, E_days, neibors, cluster, beita, beita1, force_in, force_out)
						
			elif itselves[x] == 0:					#若自身是隔离者Q
				itselves[x],E_days[x] = O_trans.Q_trans(x, len_nodes, \
				itselves, E_days, Ei, neibors, qi)
									
			elif itselves[x] == 3:					#若自身是潜伏者E
				itselves[x],E_days[x] = O_trans.E_trans(x, len_nodes, \
				itselves, E_days, Ei, neibors, sige)
				
			elif itselves[x] == 4:					#若自身是确诊者I
				itselves[x],E_days[x] = O_trans.I_trans(x, len_nodes, \
				itselves, E_days, Ei, neibors, gama, alpha)
						
		#统计节点状态
		Quarantine = 0
		Susceptible = 0
		Exposed = 0
		Infective = 0
		Recovered = 0
		Death = 0
				
		for i in itselves:
			if i == 0:
				Quarantine += 1
			elif i == 1:
				Susceptible += 1
			elif i == 3:
				Exposed += 1
			elif i == 4:
				Infective += 1
			elif i == 6:
				Recovered  += 1
			elif i == 7:
				Death += 1
		
		#更新每代的演化
		Q1[iteration]=Quarantine/len_nodes
		S1[iteration]=Susceptible/len_nodes
		E1[iteration]=Exposed/len_nodes
		I1[iteration]=Infective/len_nodes
		R1[iteration]=Recovered/len_nodes
		D1[iteration]=Death/len_nodes
		
	I_max = max(I1)
	I_no_value.append(I_max)
	
	S_last = S1[-1]
	S_no_value.append(S_last)			#终态

	Q_no=Q_no+Q1
	S_no=S_no+S1
	E_no=E_no+E1
	I_no=I_no+I1
	R_no=R_no+R1
	D_no=D_no+D1
	
Q_no=(Q_no/(timeses)).tolist()
S_no=(S_no/(timeses)).tolist()
E_no=(E_no/(timeses)).tolist()
I_no=(I_no/(timeses)).tolist()
R_no=(R_no/(timeses)).tolist()
D_no=(D_no/(timeses)).tolist()


'''
存储数据
'''
name1 = 'S_no_value'
name2 = '.json'
file_name = name1 + name2
with open(file_name, 'w') as f_obj:
	json.dump(S_no_value, f_obj)

name1 = 'I_no_value'
name2 = '.json'
file_name = name1 + name2
with open( file_name, 'w') as f_obj:
	json.dump(I_no_value, f_obj)

name1 = 'Q_no'
name2 = '.json'
file_name = name1 + name2
with open(file_name, 'w') as f_obj:
	json.dump(Q_no, f_obj)

name1 = 'S_no'
name2 = '.json'
file_name = name1 + name2
with open(file_name, 'w') as f_obj:
	json.dump(S_no, f_obj)

name1 = 'E_no'
name2 = '.json'
file_name = name1 + name2
with open(file_name, 'w') as f_obj:
	json.dump(E_no, f_obj)

name1 = 'I_no'
name2 = '.json'
file_name = name1 + name2
with open(file_name, 'w') as f_obj:
	json.dump(I_no, f_obj)

name1 = 'R_no'
name2 = '.json'
file_name = name1 + name2
with open(file_name, 'w') as f_obj:
	json.dump(R_no, f_obj)

name1 = 'D_no'
name2 = '.json'
file_name = name1 + name2
with open(file_name, 'w') as f_obj:
	json.dump(D_no, f_obj)
